@@ -64,6 +64,7 @@
 #include <syslog.h>
 #include <sys/time.h>
 #include <sys/stat.h>
+#include <sched.h>
 #include <ulogd/conffile.h>
 #include <ulogd/ulogd.h>
 #ifdef DEBUG
@@ -83,13 +84,36 @@ static char *ulogd_logfile = NULL;
 static const char *ulogd_configfile = ULOGD_CONFIGFILE;
 static const char *ulogd_pidfile = NULL;
 static int ulogd_pidfile_fd = -1;
-static FILE syslog_dummy;
+static FILE *syslog_dummy;
 
 static int info_mode = 0;
 
 static int verbose = 0;
 static int created_pidfile = 0;
 
+/*
+ * Using self-pipe trick to handle signals safely.
+ * (To avoid calling Async-Signal-Unsafe functions in signal handler.)
+ *
+ * cf. https://lwn.net/Articles/177897/
+ *
+ * Capacity of pipe(2) buffer is >= 4096.
+ */
+static int signal_channel[2] = { -1, -1 };	/* initialize with invalid fd */
+#define	SIGNAL_RX_FD	(signal_channel[0])	/* main thread select() this fd */
+#define	SIGNAL_TX_FD	(signal_channel[1])	/* signal handler writes to this fd */
+
+static void signal_handler(int signal);
+static void sigterm_handler_task(int signal);
+static void signal_handler_task(int signal);
+
+static int  signal_channel_callback(int fd, unsigned int what, void *data);
+static struct ulogd_fd	signal_channel_ulogfd = {
+	.when = ULOGD_FD_READ,
+	.cb = &signal_channel_callback,
+	.data = &signal_channel_ulogfd,
+};
+
 /* linked list for all registered plugins */
 static LLIST_HEAD(ulogd_plugins);
 /* linked list for all plugins handle */
@@ -369,6 +393,7 @@ void ulogd_register_plugin(struct ulogd_plugin *me)
 	if (strcmp(me->version, VERSION)) { 
 		ulogd_log(ULOGD_NOTICE, 
 			  "plugin `%s' has incompatible version %s\n",
+			  me->name,
 			  me->version);
 		return;
 	}
@@ -379,13 +404,87 @@ void ulogd_register_plugin(struct ulogd_plugin *me)
 				  me->name);
 			exit(EXIT_FAILURE);
 		}
-		ulogd_log(ULOGD_DEBUG, "registering plugin `%s'\n", me->name);
+		ulogd_log(ULOGD_NOTICE, "registering plugin `%s'\n", me->name);
 		llist_add(&me->list, &ulogd_plugins);
 	} else {
 		get_plugin_infos(me);
 	}
 }
 
+/***********************************************************************
+ * UTILITY FUNCTIONS FOR SIGNAL HANDLING
+ ***********************************************************************/
+static void call_signal_handler_tasks(int sig)
+{
+	/* Deliver signals */
+	switch (sig) {
+	case SIGTERM:
+	case SIGINT:
+		sigterm_handler_task(sig);
+		break;
+	case SIGHUP:
+	case SIGALRM:
+	case SIGUSR1:
+	case SIGUSR2:
+		signal_handler_task(sig);
+		break;
+	default:
+		break;
+	}
+}
+
+static int signal_channel_callback(int fd, unsigned int what, void *data)
+{
+	int	ret;
+	unsigned char c;
+
+	while ((ret = read(fd, &c, 1)) > 0) {
+		call_signal_handler_tasks((int)c);
+	}
+	return 0;
+}
+
+static int create_signal_channel(void)
+{
+	int	ret;
+	long	flags;
+
+	if ((ret = pipe(signal_channel)) < 0)
+		return ret;
+
+	/* SIGNAL_RX_FD, channel[0] */
+	if ((flags = fcntl(SIGNAL_RX_FD, F_GETFL)) < 0)
+		goto err_exit;
+	flags |= O_NONBLOCK;
+	if ( fcntl(SIGNAL_RX_FD, F_SETFL, flags) < 0 )
+		goto err_exit;
+
+	/* SIGNAL_TX_FD, channel[1] */
+	if ((flags = fcntl(SIGNAL_TX_FD, F_GETFL)) < 0)
+		goto err_exit;
+	flags |= O_NONBLOCK;
+	if ( fcntl(SIGNAL_TX_FD, F_SETFL, flags) < 0 )
+		goto err_exit;
+
+	/* register SIGNAL_RX_FD to ulogd */
+	signal_channel_ulogfd.fd = SIGNAL_RX_FD;
+	ulogd_register_fd(&signal_channel_ulogfd);
+
+	return 0;
+
+err_exit:
+	if (SIGNAL_RX_FD >= 0) {
+		(void)close(SIGNAL_RX_FD);
+		SIGNAL_RX_FD = -1;
+	}
+	if (SIGNAL_TX_FD >= 0) {
+		(void)close(SIGNAL_TX_FD);
+		SIGNAL_TX_FD = -1;
+	}
+	return -1;
+}
+
+
 /***********************************************************************
  * MAIN PROGRAM
  ***********************************************************************/
@@ -427,7 +526,7 @@ void __ulogd_log(int level, char *file, int line, const char *format, ...)
 	if (level < loglevel_ce.u.value)
 		return;
 
-	if (logfile == &syslog_dummy) {
+	if (logfile == syslog_dummy) {
 		/* FIXME: this omits the 'file' string */
 		va_start(ap, format);
 		vsyslog(ulogd2syslog_level(level), format, ap);
@@ -571,7 +670,7 @@ pluginstance_alloc_init(struct ulogd_plugin *pl, char *pi_id,
 	INIT_LLIST_HEAD(&pi->plist);
 	pi->plugin = pl;
 	pi->stack = stack;
-	memcpy(pi->id, pi_id, sizeof(pi->id));
+	strncpy(pi->id, pi_id, ULOGD_MAX_KEYLEN);
 
 	ptr = (void *)pi + sizeof(*pi);
 
@@ -629,6 +728,41 @@ static int load_plugin(const char *file)
 	return 0;
 }
 
+static int load_all_plugins(void)
+{
+	DIR *d;
+	struct dirent *dent;
+	char path[PATH_MAX];
+
+	d = opendir(ULOGD2_LIBDIR);
+	if (d == NULL) {
+		ulogd_log(ULOGD_ERROR, "load_all_plugins: opendir(%s): %s\n",
+			  ULOGD2_LIBDIR, strerror(errno));
+		return -1;
+	}
+
+	ulogd_log(ULOGD_NOTICE, "loading all plugins at %s\n", ULOGD2_LIBDIR);
+
+	while ((dent = readdir(d)) != NULL) {
+		if (strcmp(dent->d_name, ".") == 0 ||
+		    strcmp(dent->d_name, "..") == 0)
+			continue;
+
+		int len = strlen(dent->d_name);
+		if (len < 3)
+			continue;
+
+		if (strcmp(&dent->d_name[len - 3], ".so") != 0)
+			continue;
+
+		snprintf(path, sizeof(path), "%s/%s", ULOGD2_LIBDIR,
+			 dent->d_name);
+		if (load_plugin(path) != 0)
+			return -1;
+	}
+	return 0;
+}
+
 /* find an output key in a given stack, starting at 'start' */
 static struct ulogd_key *
 find_okey_in_stack(char *name,
@@ -826,6 +960,9 @@ static int create_stack(const char *option)
 	char *tok;
 	int ret;
 
+	if (llist_empty(&ulogd_plugins_handle))
+		load_all_plugins();
+
 	if (!buf) {
 		ulogd_log(ULOGD_ERROR, "");
 		ret = -ENOMEM;
@@ -925,7 +1062,6 @@ static void ulogd_main_loop(void)
 	struct timeval *next = NULL;
 
 	while (1) {
-		/* XXX: signal blocking? */
 		if (next != NULL && !timerisset(next))
 			next = ulogd_do_timer_run(&next_alarm);
 		else
@@ -950,7 +1086,7 @@ static int logfile_open(const char *name)
 		logfile = stdout;
 	} else if (!strcmp(name, "syslog")) {
 		openlog("ulogd", LOG_PID, LOG_DAEMON);
-		logfile = &syslog_dummy;
+		logfile = syslog_dummy = fopen("/dev/null", "w");
 	} else {
 		logfile = fopen(ulogd_logfile, "a");
 		if (!logfile) {
@@ -1225,7 +1361,19 @@ static void stop_stack()
 }
 
 
-static void sigterm_handler(int signal)
+/* This is a real SIGTERM, SIGINT, SIGHUP, SIGALRM, SIGUSR1, SIGUSR2 handler */
+static void signal_handler(int signal)
+{
+	unsigned char	c = (unsigned char)signal;
+
+	(void)write(SIGNAL_TX_FD, &c, 1);
+}
+
+/* This is NOT a real signal handler.
+ * It is called in ulogd_main_loop() to avoid
+ * calling Async-Signal-UnSafe functions in signal handler.
+ */
+static void sigterm_handler_task(int signal)
 {
 
 	ulogd_log(ULOGD_NOTICE, "Terminal signal received, exiting\n");
@@ -1240,7 +1388,7 @@ static void sigterm_handler(int signal)
 	unload_plugins();
 #endif
 
-	if (logfile != NULL  && logfile != stdout && logfile != &syslog_dummy) {
+	if (logfile != NULL  && logfile != stdout) {
 		fclose(logfile);
 		logfile = NULL;
 	}
@@ -1255,21 +1403,26 @@ static void sigterm_handler(int signal)
 	exit(0);
 }
 
-static void signal_handler(int signal)
+
+/* This is NOT a real signal handler.
+ * It is called in ulogd_main_loop() to avoid
+ * calling Async-Signal-UnSafe functions in signal handler.
+ */
+static void signal_handler_task(int signal)
 {
 	ulogd_log(ULOGD_NOTICE, "signal received, calling pluginstances\n");
 	
 	switch (signal) {
 	case SIGHUP:
 		/* reopen logfile */
-		if (logfile != stdout && logfile != &syslog_dummy) {
+		if (logfile != stdout && logfile != syslog_dummy) {
 			fclose(logfile);
 			logfile = fopen(ulogd_logfile, "a");
  			if (!logfile) {
 				fprintf(stderr, 
 					"ERROR: can't open logfile %s: %s\n", 
 					ulogd_logfile, strerror(errno));
-				sigterm_handler(signal);
+				sigterm_handler_task(signal);
 			}
 	
 		}
@@ -1281,6 +1434,19 @@ static void signal_handler(int signal)
 	deliver_signal_pluginstances(signal);
 }
 
+static void set_scheduler(void)
+{
+	struct sched_param schedparam;
+	int sched_type;
+
+	schedparam.sched_priority = sched_get_priority_max(SCHED_RR);
+	sched_type = SCHED_RR;
+
+	if (sched_setscheduler(0, sched_type, &schedparam) < 0)
+		fprintf(stderr, "WARNING: scheduler configuration failed:"
+			" %s\n", strerror(errno));
+}
+
 static void print_usage(void)
 {
 	printf("ulogd Version %s\n", VERSION);
@@ -1402,7 +1568,7 @@ int main(int argc, char* argv[])
 
 	if (daemonize){
 		if (daemon(0, 0) < 0) {
-			ulogd_log(ULOGD_FATAL, "can't daemonize: %s (%d)",
+			ulogd_log(ULOGD_FATAL, "can't daemonize: %s (%d)\n",
 				  errno, strerror(errno));
 			warn_and_exit(daemonize);
 		}
@@ -1464,12 +1630,18 @@ int main(int argc, char* argv[])
 		}
 	}
 
-	signal(SIGTERM, &sigterm_handler);
-	signal(SIGINT, &sigterm_handler);
+	if (create_signal_channel() < 0) {
+		ulogd_log(ULOGD_FATAL, "can't create signal channel\n");
+		warn_and_exit(daemonize);
+	}
+
+	signal(SIGTERM, &signal_handler);
+	signal(SIGINT, &signal_handler);
 	signal(SIGHUP, &signal_handler);
 	signal(SIGALRM, &signal_handler);
 	signal(SIGUSR1, &signal_handler);
 	signal(SIGUSR2, &signal_handler);
+	set_scheduler();
 
 	ulogd_log(ULOGD_INFO, 
 		  "initialization finished, entering main loop\n");
@@ -1477,6 +1649,6 @@ int main(int argc, char* argv[])
 	ulogd_main_loop();
 
 	/* hackish, but result is the same */
-	sigterm_handler(SIGTERM);	
+	sigterm_handler_task(SIGTERM);
 	return(0);
 }
